searchData={"items":[{"type":"module","title":"Mifrat","doc":"*(Mifrat is an acronym of Many Indexed Fields Random Access Table)*\n\nModule to manage an in-memory table with primary_key and secondary indexes. The objective is to\nhave a way to store complex temporary records with fast access through any indexed field.\n\nTo manage data you can use a classic functional style:\n```elixir\nget(:users, 1234)\nget(:users, :name, \"Benito Camela\")\ndelete(:users, 5432)\ninsert(:users, ...)\n# etc...\n```\n\nor you can use a pseudo query language:\n```elixir\nuse Mifrat\n\n# Get data\nmifrat_query from: :users, get: {id, name}\nmifrat_query from: :users, get: {id, name}, when: id >= 10 and id <= 15\nmifrat_query from: :users, get: {id, name, dni}, when: year, in_range: {1940,1950}\n\n# Delete data\nmifrat_query from: :users, delete_when: id < 5\nmifrat_query from: :users, delete: 5 # delete record by primary_key\nmifrat_query from: :users, delete: [1, 2, 3, 4] # delete records by primary_keys\nmifrat_query from: :users, delete_when_in_range: {1,4} # delete records by a range of primary_keys\nmifrat_query from: :users, delete_when: year, in_range: {1940, 1950}\n\n# Add/update data\n# insert_into will push new data if the primary_key does not exists, otherwise update fields\nmifrat_query insert_into: :users, values: {21, \"Cachirulo Gonzalez\", 1995, 2599945445, 1, 22845856}\n```\n\n**NOTE**: In the `when` expressions you can use ONLY guard enable functions.\n\n**TODO**: a more complex expression parser that allow `like`, `ilike`, `lower`, `upper`, and many more\ninline functions to filter records.","ref":"Mifrat.html"},{"type":"module","title":"Installation - Mifrat","doc":"```elixir\ndef deps do\n  [\n    {:mifrat_ex, \"~> 0.3.2\"}\n  ]\nend\n```","ref":"Mifrat.html#module-installation"},{"type":"function","title":"Mifrat.count/1","doc":"Return the number of records of the table.","ref":"Mifrat.html#count/1"},{"type":"function","title":"Mifrat.custom_count/3","doc":"Return the count of records that match with the `pattern` and/or `guard`. The `pattern` and the\n`guard` are strings.\nIf pattern is `:full` it will be expanded to a tuple of all fields taking the field names declared\nwith `new/2`.\n```elixir\n# Example\ncustom_count(:users, \"{_id, _name, year, _phone}\", \"year == 1985\")\n```","ref":"Mifrat.html#custom_count/3"},{"type":"function","title":"Mifrat.delete/2","doc":"Delete a record by primary_key.","ref":"Mifrat.html#delete/2"},{"type":"function","title":"Mifrat.delete/3","doc":"Delete one or more records by a secondary_key.","ref":"Mifrat.html#delete/3"},{"type":"function","title":"Mifrat.delete_all/1","doc":"WARNING!!! This function remove physically every record in the table.","ref":"Mifrat.html#delete_all/1"},{"type":"function","title":"Mifrat.delete_list/2","doc":"Delete many records referenced with its primary key.","ref":"Mifrat.html#delete_list/2"},{"type":"function","title":"Mifrat.delete_range/3","doc":"Delete a bunch of records referenced by a range of its primary key.","ref":"Mifrat.html#delete_range/3"},{"type":"function","title":"Mifrat.delete_range/4","doc":"Delete a bunch of records referenced by a range of one of its secondary indexes keys.","ref":"Mifrat.html#delete_range/4"},{"type":"function","title":"Mifrat.destroy/1","doc":"Delete the table and his indexes. No data survive this process. If the table has enable\n**autosave**, before delete the data will be flushed to disk (see `new/3`).","ref":"Mifrat.html#destroy/1"},{"type":"type","title":"Mifrat.field_range/0","doc":"","ref":"Mifrat.html#t:field_range/0"},{"type":"function","title":"Mifrat.get/2","doc":"Return the full record referenced by the primary_key.","ref":"Mifrat.html#get/2"},{"type":"function","title":"Mifrat.get/4","doc":"Return one o more records using a secondary index key. Return the the auxiliary indexed record or\nthe full main record of the table.\nOption `:return` can be:\n  - `return: :records`: return a list of full records from the table\n  - `return: :keys`: return a list of auxiliary secondary index record (`{{: , key}, primary_key}`)","ref":"Mifrat.html#get/4"},{"type":"function","title":"Mifrat.get_range/5","doc":"Delete a bunch of records referenced by a range of one of its primary key or its secondary indexes\nkeys.\nIf you have a table like this:\n```elixir\nnew(:users, [\n  id: :primary_key\n  name: :unindexed,\n  year: :indexed_non_uniq,\n  phone: :indexed\n], [\n  ...\n])\n```\nYou can get a range of records using the primary key:\n```elixir\nget_range(table, :id, 100, 200)  # get the records with id >= 100 and <= 200\n```\nor get a range of records using the key of a secondary index:\n```elixir\nget_range(table, :year, 1940, 1950)  # get the records with year >= 1940 and <= 1950\n```\nJust as in `get/3` you can get a list of full records from the table or a list of auxiliary\nsecondary index record (return: :records or return: :keys)","ref":"Mifrat.html#get_range/5"},{"type":"macro","title":"Mifrat.imft_query/1","doc":"","ref":"Mifrat.html#imft_query/1"},{"type":"function","title":"Mifrat.insert/2","doc":"Insert or update a record. If the primary key does not exists it insert, otherwise update.\nThe fields values must follow the order declared with `new/2` or `new/3`. It can be a list or a\ntuple.\n```elixir\ninsert(:users, {1, \"Jorge Luis Borges\", 1899, 542915040798})\n```\nor\n```elixir\ninsert(:users, [1, \"Jorge Luis Borges\", 1899, 542915040798])\n```","ref":"Mifrat.html#insert/2"},{"type":"function","title":"Mifrat.list_table_indexes/2","doc":"","ref":"Mifrat.html#list_table_indexes/2"},{"type":"function","title":"Mifrat.load/1","doc":"Load the table from file in pathname.","ref":"Mifrat.html#load/1"},{"type":"function","title":"Mifrat.map_to_record/2","doc":"Convert a map into a tuple record.","ref":"Mifrat.html#map_to_record/2"},{"type":"macro","title":"Mifrat.mifrat_query/1","doc":"Do it easy, make queries with a more natural language.\n\nThis macro define a pseudo query languate to get/delete/insert data in the table.\nIf you have a table `:users` with this struct:\n```elixir\n[\n  id: :primary_key,\n  name: :unindexed,\n  year: :indexed_non_uniq,\n  phone: :indexed,\n  category: :indexed_non_uniq,\n  dni: :indexed\n]\n```\n\nYou could do:\n```elixir\n# Get data\nmifrat_query from: :users, get: {id, name}\nmifrat_query from: :users, get: {id, name}, when: id >= 10 and id <= 15\nmifrat_query from: :users, get: {id, name, dni}, when: year, in_range: {1940,1950}\n\n# Delete data\nmifrat_query from: :users, delete_when: id < 5\nmifrat_query from: :users, delete: 5 # delete record by primary_key\nmifrat_query from: :users, delete: [1, 2, 3, 4] # delete records by primary_keys\nmifrat_query from: :users, delete_when_in_range: {1,4} # delete records by a range of primary_keys\nmifrat_query from: :users, delete_when: year, in_range: {1940, 1950}\n\n# Add/update data\n# insert_into will push new data if the primary_key does not exists, otherwise update fields\nmifrat_query insert_into: :users, values: {21, \"Cachirulo Gonzalez\", 1995, 2599945445, 1, 22845856}\n```","ref":"Mifrat.html#mifrat_query/1"},{"type":"function","title":"Mifrat.new/3","doc":"Create a table. The fields are defined with a keyword list; each pair has a field name (the key) and an index\ntype (the value). The order of the fields is important and the primary key must be the first.\n```elixir\nnew(:users, [\n  id: :primary_key\n  name: :unindexed,\n  year: :indexed_non_uniq,\n  phone: :indexed\n], [\n  ...opts...\n])\n```\nThe fields type available are:\n- `:primary_key`: just one field can has this type and it is the main index of the table.\n- `:indexed`: the field will has an auxiliary uniq index.\n- `:indexed_non_uniq`: the field will has an auxiliary non uniq index.\n- `:unindexed`: the field won't be indexed, it is just data.\n\nThe options available are:\n- `:_autosave`: Force the flush of the table to disk every `:period` (see below) in `:path` file\n(see below). If true make mandatory `:path`. Default `false`.\n- `:period`: Set how often autosave will flush to disk. It is a value in miliseconds. Default\n`300_000` (5 minutes).\n- `:path`: The path filename where the table will be flushed. Ignored if `autosave: false`.\n- `:initial_load`: If `true` and `autosave: true` will try to load from `:path` the table. Ignored\nif `autosave: false`. Default is `true`.","ref":"Mifrat.html#new/3"},{"type":"function","title":"Mifrat.record_to_map/2","doc":"If the first argument is a tuple, this function try to convert the tuple to a map. For that, the function\nuse the information about fields stored in the `table`. If the first argument is not a tuple, the function\nwill return just the argument passed. This is useful specially when the tuple record to convert come\nfrom a `get/n` call because that function can return a tuple or :not_found.","ref":"Mifrat.html#record_to_map/2"},{"type":"function","title":"Mifrat.reindex/1","doc":"Clear every secondary indexes and rebuild every one again from zero.","ref":"Mifrat.html#reindex/1"},{"type":"function","title":"Mifrat.save/1","doc":"Flush the table to disk in the file configured for autosave.","ref":"Mifrat.html#save/1"},{"type":"function","title":"Mifrat.store/2","doc":"Flush the table to disk in the file pointed by pathname.","ref":"Mifrat.html#store/2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}